@@ -16,16 +16,16 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "groonga_in.h"
+#include "grn.h"
 #include <string.h>
-#include "token.h"
-#include "ctx_impl.h"
-#include "pat.h"
-#include "plugin_in.h"
-#include "snip.h"
-#include "output.h"
-#include "normalizer_in.h"
-#include "ctx_impl_mrb.h"
+#include "grn_tokenizers.h"
+#include "grn_ctx_impl.h"
+#include "grn_pat.h"
+#include "grn_plugin.h"
+#include "grn_snip.h"
+#include "grn_output.h"
+#include "grn_normalizer.h"
+#include "grn_ctx_impl_mrb.h"
 #include <stdio.h>
 #include <stdarg.h>
 #include <time.h>
@@ -493,23 +493,18 @@ grn_ctx_impl_init(grn_ctx *ctx)
 
   ctx->impl->finalizer = NULL;
 
-  ctx->impl->op = GRN_OP_T0LVL;
-  ctx->impl->n_entries = 0;
-  ctx->impl->cur = NULL;
-  ctx->impl->str_end = NULL;
-  ctx->impl->batchmode = 0;
-  ctx->impl->inbuf = NULL;
   ctx->impl->com = NULL;
   ctx->impl->outbuf = grn_obj_open(ctx, GRN_BULK, 0, 0);
-  ctx->impl->output = NULL /* grn_ctx_concat_func */;
+  ctx->impl->output = NULL;
   ctx->impl->data.ptr = NULL;
   ctx->impl->tv.tv_sec = 0;
   ctx->impl->tv.tv_nsec = 0;
-  GRN_TEXT_INIT(&ctx->impl->subbuf, 0);
   ctx->impl->edge = NULL;
   grn_loader_init(&ctx->impl->loader);
   ctx->impl->plugin_path = NULL;
 
+  GRN_TEXT_INIT(&ctx->impl->query_log_buf, 0);
+
   ctx->impl->previous_errbuf[0] = '\0';
   ctx->impl->n_same_error_messages = 0;
 
@@ -526,15 +521,6 @@ grn_ctx_set_next_expr(grn_ctx *ctx, grn_obj *expr)
   ctx->impl->qe_next = expr;
 }
 
-void
-grn_ctx_impl_err(grn_ctx *ctx)
-{
-  if (ctx->impl) {
-    ctx->impl->cur = ctx->impl->str_end;
-    ctx->impl->op = GRN_OP_ERR0;
-  }
-}
-
 static void
 grn_ctx_impl_clear_n_same_error_mssagges(grn_ctx *ctx)
 {
@@ -590,11 +576,6 @@ grn_ctx_init(grn_ctx *ctx, int flags)
   ctx->seqno2 = 0;
   ctx->subno = 0;
   ctx->impl = NULL;
-  if (flags & GRN_CTX_USE_QL) {
-    grn_ctx_impl_init(ctx);
-    if (ERRP(ctx, GRN_ERROR)) { return ctx->rc; }
-    if (flags & GRN_CTX_BATCH_MODE) { ctx->impl->batchmode = 1; }
-  }
   ctx->user_data.ptr = NULL;
   CRITICAL_SECTION_ENTER(grn_glock);
   ctx->next = grn_gctx.next;
@@ -672,8 +653,8 @@ grn_ctx_fin(grn_ctx *ctx)
     }
     GRN_OBJ_FIN(ctx, &ctx->impl->names);
     GRN_OBJ_FIN(ctx, &ctx->impl->levels);
+    GRN_OBJ_FIN(ctx, &ctx->impl->query_log_buf);
     rc = grn_obj_close(ctx, ctx->impl->outbuf);
-    rc = grn_bulk_fin(ctx, &ctx->impl->subbuf);
     {
       grn_hash **vp;
       grn_obj *value;
@@ -1121,7 +1102,6 @@ grn_query_logger_pass(grn_ctx *ctx, unsigned int flag)
 #define TIMESTAMP_BUFFER_SIZE    TBUFSIZE
 /* 8+a(%p) + 1(|) + 1(mark) + 15(elapsed time) = 25+a */
 #define INFO_BUFFER_SIZE         40
-#define MESSAGE_BUFFER_SIZE      MBUFSIZE
 
 void
 grn_query_logger_put(grn_ctx *ctx, unsigned int flag, const char *mark,
@@ -1129,7 +1109,7 @@ grn_query_logger_put(grn_ctx *ctx, unsigned int flag, const char *mark,
 {
   char timestamp[TIMESTAMP_BUFFER_SIZE];
   char info[INFO_BUFFER_SIZE];
-  char message[MESSAGE_BUFFER_SIZE];
+  grn_obj *message = &ctx->impl->query_log_buf;
 
   if (!current_query_logger.log) {
     return;
@@ -1160,13 +1140,15 @@ grn_query_logger_put(grn_ctx *ctx, unsigned int flag, const char *mark,
 
   {
     va_list args;
+
     va_start(args, format);
-    vsnprintf(message, MESSAGE_BUFFER_SIZE - 1, format, args);
+    GRN_BULK_REWIND(message);
+    grn_text_vprintf(ctx, message, format, args);
     va_end(args);
-    message[MESSAGE_BUFFER_SIZE - 1] = '\0';
+    GRN_TEXT_PUTC(ctx, message, '\0');
   }
 
-  current_query_logger.log(ctx, flag, timestamp, info, message,
+  current_query_logger.log(ctx, flag, timestamp, info, GRN_TEXT_VALUE(message),
                            current_query_logger.user_data);
 }
 
@@ -1300,8 +1282,8 @@ grn_init(void)
     GRN_LOG(ctx, GRN_LOG_ALERT, "grn_normalizer_init failed (%d)", rc);
     return rc;
   }
-  if ((rc = grn_token_init())) {
-    GRN_LOG(ctx, GRN_LOG_ALERT, "grn_token_init failed (%d)", rc);
+  if ((rc = grn_tokenizers_init())) {
+    GRN_LOG(ctx, GRN_LOG_ALERT, "grn_tokenizers_init failed (%d)", rc);
     return rc;
   }
   /*
@@ -1398,7 +1380,7 @@ grn_fin(void)
   }
   query_logger_fin(ctx);
   grn_cache_fin();
-  grn_token_fin();
+  grn_tokenizers_fin();
   grn_normalizer_fin();
   grn_plugins_fin();
   grn_io_fin();
@@ -1446,16 +1428,6 @@ grn_ctx_get_command_version(grn_ctx *ctx)
   }
 }
 
-const char *
-grn_ctx_get_mime_type(grn_ctx *ctx)
-{
-  if (ctx->impl) {
-    return ctx->impl->mime_type;
-  } else {
-    return NULL;
-  }
-}
-
 grn_rc
 grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version)
 {
@@ -1474,6 +1446,40 @@ grn_ctx_set_command_version(grn_ctx *ctx, grn_command_version version)
   }
 }
 
+grn_content_type
+grn_ctx_get_output_type(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->output_type;
+  } else {
+    return GRN_CONTENT_NONE;
+  }
+}
+
+grn_rc
+grn_ctx_set_output_type(grn_ctx *ctx, grn_content_type type)
+{
+  grn_rc rc = GRN_SUCCESS;
+
+  if (ctx->impl) {
+    ctx->impl->output_type = type;
+  } else {
+    rc = GRN_INVALID_ARGUMENT;
+  }
+
+  return rc;
+}
+
+const char *
+grn_ctx_get_mime_type(grn_ctx *ctx)
+{
+  if (ctx->impl) {
+    return ctx->impl->mime_type;
+  } else {
+    return NULL;
+  }
+}
+
 long long int
 grn_ctx_get_match_escalation_threshold(grn_ctx *ctx)
 {
@@ -1918,17 +1924,6 @@ grn_ctx_recv(grn_ctx *ctx, char **str, unsigned int *str_len, int *flags)
     } else {
       grn_obj *buf = ctx->impl->outbuf;
       unsigned int head = 0, tail = GRN_BULK_VSIZE(buf);
-      /*
-      unsigned int *offsets = (unsigned int *) GRN_BULK_HEAD(&ctx->impl->subbuf);
-      int npackets = GRN_BULK_VSIZE(&ctx->impl->subbuf) / sizeof(unsigned int);
-      if (npackets < ctx->impl->bufcur) {
-        ERR(GRN_INVALID_ARGUMENT, "invalid argument");
-        goto exit;
-      }
-      head = ctx->impl->bufcur ? offsets[ctx->impl->bufcur - 1] : 0;
-      tail = ctx->impl->bufcur < npackets ? offsets[ctx->impl->bufcur] : GRN_BULK_VSIZE(buf);
-      *flags = ctx->impl->bufcur++ < npackets ? GRN_CTX_MORE : 0;
-      */
       *str = GRN_BULK_HEAD(buf) + head;
       *str_len = tail - head;
       GRN_BULK_REWIND(ctx->impl->outbuf);
@@ -1940,15 +1935,6 @@ exit :
   GRN_API_RETURN(0);
 }
 
-void
-grn_ctx_concat_func(grn_ctx *ctx, int flags, void *dummy)
-{
-  if (ctx && ctx->impl && (flags & GRN_CTX_MORE)) {
-    unsigned int size = GRN_BULK_VSIZE(ctx->impl->outbuf);
-    grn_bulk_write(ctx, &ctx->impl->subbuf, (char *) &size, sizeof(unsigned int));
-  }
-}
-
 void
 grn_ctx_stream_out_func(grn_ctx *ctx, int flags, void *stream)
 {
@@ -2258,7 +2244,7 @@ grn_ctx_alloc(grn_ctx *ctx, size_t size, int flags,
     if (size > GRN_CTX_SEGMENT_SIZE) {
       uint64_t npages = (size + (grn_pagesize - 1)) / grn_pagesize;
       if (npages >= (1LL<<32)) {
-        MERR("too long request size=%zu", size);
+        MERR("too long request size=%" GRN_FMT_SIZE, size);
         goto exit;
       }
       for (i = 0, mi = ctx->impl->segs;; i++, mi++) {
@@ -2463,7 +2449,7 @@ grn_ctx_alloc_lifo(grn_ctx *ctx, size_t size,
     if (size > GRN_CTX_SEGMENT_SIZE) {
       uint64_t npages = (size + (grn_pagesize - 1)) / grn_pagesize;
       if (npages >= (1LL<<32)) {
-        MERR("too long request size=%zu", size);
+        MERR("too long request size=%" GRN_FMT_SIZE, size);
         return NULL;
       }
       for (;;) {
@@ -2652,7 +2638,7 @@ grn_malloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
       grn_alloc_info_add(res);
     } else {
       if (!(res = malloc(size))) {
-        MERR("malloc fail (%zu)=%p (%s:%d) <%d>",
+        MERR("malloc fail (%" GRN_FMT_SIZE ")=%p (%s:%d) <%d>",
              size, res, file, line, alloc_count);
       } else {
         GRN_ADD_ALLOC_COUNT(1);
@@ -2709,7 +2695,8 @@ grn_realloc_default(grn_ctx *ctx, void *ptr, size_t size, const char* file, int
   if (size) {
     if (!(res = realloc(ptr, size))) {
       if (!(res = realloc(ptr, size))) {
-        MERR("realloc fail (%p,%zu)=%p (%s:%d) <%d>", ptr, size, res, file, line, alloc_count);
+        MERR("realloc fail (%p,%" GRN_FMT_SIZE ")=%p (%s:%d) <%d>",
+             ptr, size, res, file, line, alloc_count);
         return NULL;
       }
     }
@@ -2796,7 +2783,8 @@ grn_realloc_fail(grn_ctx *ctx, void *ptr, size_t size, const char* file, int lin
   if (grn_fail_malloc_check(size, file, line, func)) {
     return grn_realloc_default(ctx, ptr, size, file, line, func);
   } else {
-    MERR("fail_realloc (%p,%zu) (%s:%d@%s) <%d>", ptr, size, file, line, func, alloc_count);
+    MERR("fail_realloc (%p,%" GRN_FMT_SIZE ") (%s:%d@%s) <%d>",
+         ptr, size, file, line, func, alloc_count);
     return NULL;
   }
 }
