@@ -18,10 +18,10 @@
 
 #include <string.h>
 
-#include "ctx_impl_mrb.h"
-#include "ctx_impl.h"
+#include "grn_ctx_impl_mrb.h"
+#include "grn_ctx_impl.h"
 
-#include "mrb.h"
+#include "grn_mrb.h"
 #include "mrb/mrb_error.h"
 #include "mrb/mrb_id.h"
 #include "mrb/mrb_operator.h"
@@ -39,6 +39,21 @@
 #include "mrb/mrb_procedure.h"
 
 #ifdef GRN_WITH_MRUBY
+static mrb_value
+mrb_kernel_load(mrb_state *mrb, mrb_value self)
+{
+  grn_ctx *ctx = (grn_ctx *)mrb->ud;
+  char *path;
+
+  mrb_get_args(mrb, "z", &path);
+
+  grn_mrb_load(ctx, path);
+
+  grn_mrb_ctx_check(mrb);
+
+  return mrb_true_value();
+}
+
 static void
 grn_ctx_impl_mrb_init_bindings(grn_ctx *ctx)
 {
@@ -47,6 +62,9 @@ grn_ctx_impl_mrb_init_bindings(grn_ctx *ctx)
   mrb->ud = ctx;
   ctx->impl->mrb.module = mrb_define_module(mrb, "Groonga");
 
+  mrb_define_method(mrb, mrb->kernel_module,
+                    "load", mrb_kernel_load, MRB_ARGS_REQ(1));
+
   grn_mrb_load(ctx, "backtrace_entry.rb");
 
   grn_mrb_error_init(ctx);
@@ -79,9 +97,12 @@ grn_ctx_impl_mrb_init(grn_ctx *ctx)
   grn_mruby_enabled = getenv("GRN_MRUBY_ENABLED");
   if (grn_mruby_enabled && strcmp(grn_mruby_enabled, "no") == 0) {
     ctx->impl->mrb.state = NULL;
+    ctx->impl->mrb.base_directory[0] = '\0';
     ctx->impl->mrb.module = NULL;
+    ctx->impl->mrb.object_class = NULL;
   } else {
     ctx->impl->mrb.state = mrb_open();
+    ctx->impl->mrb.base_directory[0] = '\0';
     grn_ctx_impl_mrb_init_bindings(ctx);
     grn_ctx_impl_mrb_init_eval(ctx);
   }
