@@ -16,8 +16,9 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "mrb.h"
-#include "ctx_impl.h"
+#include "grn_mrb.h"
+#include "grn_ctx_impl.h"
+#include "grn_util.h"
 
 #ifdef GRN_WITH_MRUBY
 # include <mruby/proc.h>
@@ -28,7 +29,7 @@
 #define BUFFER_SIZE 2048
 
 #ifdef GRN_WITH_MRUBY
-#ifdef WIN32
+# ifdef WIN32
 static char *win32_ruby_scripts_dir = NULL;
 static char win32_ruby_scripts_dir_buffer[PATH_MAX];
 static const char *
@@ -37,7 +38,6 @@ grn_mrb_get_system_ruby_scripts_dir(void)
   if (!win32_ruby_scripts_dir) {
     const char *base_dir;
     const char *relative_path = GRN_RELATIVE_RUBY_SCRIPTS_DIR;
-    char *path;
     size_t base_dir_length;
 
     base_dir = grn_win32_base_dir();
@@ -50,25 +50,26 @@ grn_mrb_get_system_ruby_scripts_dir(void)
   return win32_ruby_scripts_dir;
 }
 
-#else /* WIN32 */
+# else /* WIN32 */
 static const char *
 grn_mrb_get_system_ruby_scripts_dir(void)
 {
   return GRN_RUBY_SCRIPTS_DIR;
 }
-#endif /* WIN32 */
+# endif /* WIN32 */
 
-static FILE *
-grn_mrb_open_script(grn_ctx *ctx, const char *path)
+static grn_bool
+grn_mrb_expand_script_path(grn_ctx *ctx, const char *path, char *expanded_path)
 {
   const char *ruby_scripts_dir;
   char dir_last_char;
-  char expanded_path[PATH_MAX];
   int path_length, max_path_length;
-  FILE *script_file = NULL;
 
   if (path[0] == '/') {
     expanded_path[0] = '\0';
+  } else if (path[0] == '.' && path[1] == '/') {
+    strcpy(expanded_path, ctx->impl->mrb.base_directory);
+    strcat(expanded_path, "/");
   } else {
     ruby_scripts_dir = getenv("GRN_RUBY_SCRIPTS_DIR");
     if (!ruby_scripts_dir) {
@@ -89,27 +90,20 @@ grn_mrb_open_script(grn_ctx *ctx, const char *path)
         "script path is too long: %d (max: %d) <%s%s>",
         path_length, max_path_length,
         expanded_path, path);
-    return NULL;
+    return GRN_FALSE;
   }
 
   strcat(expanded_path, path);
-  script_file = fopen(expanded_path, "r");
-  if (!script_file) {
-    char message[BUFFER_SIZE];
-    snprintf(message, BUFFER_SIZE - 1,
-             "fopen: failed to open mruby script file: <%s>", path);
-    SERR(message);
-    return NULL;
-  }
 
-  return script_file;
+  return GRN_TRUE;
 }
 
 mrb_value
 grn_mrb_load(grn_ctx *ctx, const char *path)
 {
-  mrb_state *mrb = ctx->impl->mrb.state;
-  int n;
+  grn_mrb_data *data = &(ctx->impl->mrb);
+  mrb_state *mrb = data->state;
+  char expanded_path[PATH_MAX];
   FILE *file;
   mrb_value result;
   struct mrb_parser_state *parser;
@@ -118,32 +112,59 @@ grn_mrb_load(grn_ctx *ctx, const char *path)
     return mrb_nil_value();
   }
 
-  file = grn_mrb_open_script(ctx, path);
+  if (!grn_mrb_expand_script_path(ctx, path, expanded_path)) {
+    return mrb_nil_value();
+  }
+
+  file = fopen(expanded_path, "r");
   if (!file) {
+    char message[BUFFER_SIZE];
     mrb_value exception;
+    snprintf(message, BUFFER_SIZE - 1,
+             "fopen: failed to open mruby script file: <%s>", expanded_path);
+    SERR(message);
     exception = mrb_exc_new(mrb, E_ARGUMENT_ERROR,
                             ctx->errbuf, strlen(ctx->errbuf));
     mrb->exc = mrb_obj_ptr(exception);
     return mrb_nil_value();
   }
 
-  parser = mrb_parse_file(mrb, file, NULL);
+  {
+    char current_base_directory[PATH_MAX];
+    char *last_directory;
+
+    strcpy(current_base_directory, data->base_directory);
+    strcpy(data->base_directory, expanded_path);
+    last_directory = strrchr(data->base_directory, '/');
+    if (last_directory) {
+      last_directory[0] = '\0';
+    }
+
+  parser = mrb_parser_new(mrb);
+  mrb_parser_set_filename(parser, expanded_path);
+  parser->s = parser->send = NULL;
+  parser->f = file;
+  mrb_parser_parse(parser, NULL);
   fclose(file);
 
-  n = mrb_generate_code(mrb, parser);
-  result = mrb_run(mrb,
-                   mrb_proc_new(mrb, mrb->irep[n]),
-                   mrb_top_self(mrb));
+  {
+    struct RProc *proc;
+    proc = mrb_generate_code(mrb, parser);
+    result = mrb_toplevel_run(mrb, proc);
+  }
   mrb_parser_free(parser);
 
+  strcpy(data->base_directory, current_base_directory);
+  }
+
   return result;
 }
 
 mrb_value
 grn_mrb_eval(grn_ctx *ctx, const char *script, int script_length)
 {
-  mrb_state *mrb = ctx->impl->mrb.state;
-  int n;
+  grn_mrb_data *data = &(ctx->impl->mrb);
+  mrb_state *mrb = data->state;
   mrb_value result;
   struct mrb_parser_state *parser;
 
@@ -155,10 +176,16 @@ grn_mrb_eval(grn_ctx *ctx, const char *script, int script_length)
     script_length = strlen(script);
   }
   parser = mrb_parse_nstring(mrb, script, script_length, NULL);
-  n = mrb_generate_code(mrb, parser);
-  result = mrb_run(mrb,
-                   mrb_proc_new(mrb, mrb->irep[n]),
-                   mrb_top_self(mrb));
+  {
+    struct RProc *proc;
+    struct RClass *eval_context_class;
+    mrb_value eval_context;
+
+    proc = mrb_generate_code(mrb, parser);
+    eval_context_class = mrb_class_get_under(mrb, data->module, "EvalContext");
+    eval_context = mrb_obj_new(mrb, eval_context_class, 0, NULL);
+    result = mrb_context_run(mrb, proc, eval_context, 0);
+  }
   mrb_parser_free(parser);
 
   return result;
@@ -168,6 +195,8 @@ grn_rc
 grn_mrb_to_grn(grn_ctx *ctx, mrb_value mrb_object, grn_obj *grn_object)
 {
   grn_rc rc = GRN_SUCCESS;
+  grn_mrb_data *data = &(ctx->impl->mrb);
+  mrb_state *mrb = data->state;
 
   switch (mrb_type(mrb_object)) {
   case MRB_TT_FALSE :
@@ -192,6 +221,18 @@ grn_mrb_to_grn(grn_ctx *ctx, mrb_value mrb_object, grn_obj *grn_object)
                  RSTRING_PTR(mrb_object),
                  RSTRING_LEN(mrb_object));
     break;
+  case MRB_TT_SYMBOL :
+    {
+      const char *name;
+      int name_length;
+
+      grn_obj_reinit(ctx, grn_object, GRN_DB_TEXT, 0);
+      GRN_BULK_REWIND(grn_object);
+      GRN_TEXT_PUTC(ctx, grn_object, ':');
+      name = mrb_sym2name_len(mrb, mrb_symbol(mrb_object), &name_length);
+      GRN_TEXT_PUT(ctx, grn_object, name, name_length);
+    }
+    break;
   default :
     rc = GRN_INVALID_ARGUMENT;
     break;
