@@ -16,7 +16,7 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
-#include "../ctx_impl.h"
+#include "../grn_ctx_impl.h"
 
 #ifdef GRN_WITH_MRUBY
 #include <mruby.h>
@@ -55,13 +55,20 @@ grn_mrb_class_from_grn_obj(mrb_state *mrb, grn_obj *object)
   case GRN_PROC :
     klass = mrb_class_get_under(mrb, data->module, "Procedure");
     break;
+  case GRN_VOID :
+    klass = mrb_class_get_under(mrb, data->module, "Void");
+    break;
   default :
     break;
   }
 
   if (!klass) {
-    mrb_raisef(mrb, E_ARGUMENT_ERROR,
-               "can't find class for object type: %#x", object->header.type);
+#define BUFFER_SIZE 1024
+    char buffer[BUFFER_SIZE];
+    snprintf(buffer, BUFFER_SIZE - 1,
+             "can't find class for object type: %#x", object->header.type);
+    mrb_raise(mrb, E_ARGUMENT_ERROR, buffer);
+#undef BUFFER_SIZE
   }
 
   return klass;
