@@ -19,13 +19,14 @@
 #ifdef WIN32
 # define GROONGA_MAIN
 #endif /* WIN32 */
-#include "lib/groonga_in.h"
+#include <grn.h>
+
+#include <grn_com.h>
+#include <grn_ctx_impl.h>
+#include <grn_proc.h>
+#include <grn_db.h>
+#include <grn_util.h>
 
-#include "lib/com.h"
-#include "lib/ctx_impl.h"
-#include "lib/proc.h"
-#include "lib/db.h"
-#include "lib/util.h"
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -48,6 +49,10 @@
 # include <sys/sysctl.h>
 #endif /* HAVE_SYS_SYSCTL_H */
 
+#ifdef HAVE_IO_H
+# include <io.h>
+#endif /* HAVE_IO_H */
+
 #ifdef HAVE__STRNICMP
 # define strncasecmp(s1,s2,n) _strnicmp(s1,s2,n)
 #endif /* HAVE__STRNICMP */
@@ -691,8 +696,12 @@ h_output(grn_ctx *ctx, int flags, void *arg)
     wsabufs[2].len = GRN_TEXT_LEN(outbuf);
     wsabufs[3].buf = GRN_TEXT_VALUE(&foot);
     wsabufs[3].len = GRN_TEXT_LEN(&foot);
-    if (WSASend(fd, wsabufs, 4, &ret, 0, NULL, NULL) == SOCKET_ERROR) {
-      SERR("WSASend");
+    {
+      DWORD sent;
+      if (WSASend(fd, wsabufs, 4, &sent, 0, NULL, NULL) == SOCKET_ERROR) {
+        SERR("WSASend");
+      }
+      ret = sent;
     }
 #else /* WIN32 */
     struct iovec msg_iov[4];
@@ -1653,7 +1662,7 @@ check_rlimit_nofile(grn_ctx *ctx)
 #endif /* WIN32 */
 }
 
-static void * CALLBACK
+static grn_thread_func_result CALLBACK
 h_worker(void *arg)
 {
   ht_context hc;
@@ -1684,7 +1693,7 @@ exit :
   MUTEX_UNLOCK(q_mutex);
   GRN_LOG(&grn_gctx, GRN_LOG_NOTICE, "thread end (%d/%d)", nfthreads, nthreads);
   grn_ctx_fin(ctx);
-  return NULL;
+  return GRN_THREAD_FUNC_RETURN_VALUE;
 }
 
 static void
@@ -1729,7 +1738,7 @@ h_server(char *path)
   return exit_code;
 }
 
-static void * CALLBACK
+static grn_thread_func_result CALLBACK
 g_worker(void *arg)
 {
   GRN_LOG(&grn_gctx, GRN_LOG_NOTICE, "thread start (%d/%d)", nfthreads, nthreads + 1);
@@ -1790,7 +1799,7 @@ exit :
   nthreads--;
   MUTEX_UNLOCK(q_mutex);
   GRN_LOG(&grn_gctx, GRN_LOG_NOTICE, "thread end (%d/%d)", nfthreads, nthreads);
-  return NULL;
+  return GRN_THREAD_FUNC_RETURN_VALUE;
 }
 
 static void
@@ -2274,11 +2283,11 @@ show_version(void)
 #ifdef GRN_WITH_MESSAGE_PACK
   printf(",msgpack");
 #endif
-#ifndef NO_ZLIB
+#ifdef GRN_WITH_ZLIB
   printf(",zlib");
 #endif
-#ifndef NO_LZO
-  printf(",lzo");
+#ifdef GRN_WITH_LZ4
+  printf(",lz4");
 #endif
 #ifdef USE_KQUEUE
   printf(",kqueue");
